PROGRAM_NAME='Main'

// System Type : Netlinx

// FILE CREATED ON: 9/12/2002 11:06PM
//              BY: 3rd Party Systems
//                  5316 91st Ave SE
//                  Snohomish, WA  98290
//                  (360) 568-9522
//                  www.thrdParty.com
// FILE REVISION: Rev 1.0
// REVISION DATE: 1/10/2003 8:48PM

// **********************  Notice  ********************************************
// This '.axs' file is the sample program that uses the 'RaneRPM.axi' include file.
// This file, along with the included Touch panel file, 'RaneTestSO#REV0.tpd' will
// use the Functions and Calls defined and implemented in the 'RaneRPM.axi' file.  
// Please note that there are other ways to implement the same functionality 
// demonstrated here. The method used in the Include file has some drawbacks when
// controlling multiple units in a high usage environment.  Please see the notes 
// in the include file for these explanations.
//
//  This documentation assumes you have a good understanding of the NetLinx
//  programming language.  It is not intended to be a tutorial for the NetLinx
//  language.  Please refer to the proper AMX documentation for explanations
//  of specific language elements.
//
// The designation, '** Required' , precedes each element that is required in
// your program file.


// compiler define statements - these defines are not case sensitive

// The debug define sends the command or information strings
// to the master port defined below.  You may define the debug port as
// any port you desire.  For instance you could send the debug info to an IP,
// or standard serial, port.
#define debug

//          DEVICE NUMBER DEFINITIONS GO BELOW
DEFINE_DEVICE

// ** Required
dvRPM_CLIENT    = 0:3:0 // this is where you declare a port for each RPM unit
                        // used in your system.  This is required in your Main 
                        // program file.  You can also use the AMX first_local_port
                        // function to automatically get the first available port.
                                                
dvDEBUG         = 0:0:0 // for sending info to the master port wherever the
                        // if_defined debug statement is used

dvPNL           = 128:1:0


//      *******  CONSTANT DEFINITIONS GO BELOW  *******

DEFINE_CONSTANT

// ** Required
UDP_2               = 3  // AMX has several Ethernet protocol definitions.  
                         // 1 = TCP/IP       2 = UDP     3 = 2-Way UDP
                         // The 2-way UDP was not in any AMX documentation
                         // Rane Drag Net devices use 2-way UDP for packet verification
                                                        

// ** Required
RPM_ADDR    = '192.168.48.13'  // You must have this assignment for each RPM
                               // unit in your system. Assign each unit a unique
                               // name and address.

L_LEVELBAR = 1    R_LEVELBAR = 2    // These are the level bar assignments in the
    bLMUTE = 1        bRMUTE = 2    // Touch Panel file.  You may or may not use level
                                    // bars in your Panel file.
    

//     *******  DATA TYPE DEFINITIONS GO BELOW   ********
DEFINE_TYPE


//  ***********  Include file delcarations      **********
// ** Required
#include 'raneRPM.axi'  // This file will be included in the program by the compiler
                        // The exact file name could be different if you create
                        // your own include file.


//  ***********  VARIABLE DEFINITIONS GO BELOW  *********
DEFINE_VARIABLE

Persistent chanLink = 0 // variable used to determine if two channels are linked
                        // to one volume control.  This is for demo purposes, and
                        // not required.


// This demo program includes only a small number of the Processing Blocks available in  
// Rane Drag Net products.  Included are the "Level" block, "1 of N Select", and
// "Preset" Selection. The following declarations are defined in the 'Define_Type'
// section of the Include file and are described in more detail there.
//
// PID - What's a Rane PID? In Rane's Drag Net software, each Processing Block is assigned 
// a Parameter IDentification value. View each block's PID by drawing the Processing Map in Drag Net,
// Save it, then select Generate Report from Drag Net's File menu.
//
//
// These Level block Types take six elements:
//  1)  The PID which is assigned by the RPM/Drag Net unit and can be found in the Drag Net 
//          XML Report
//  2)  The current volume level
//  3)  The current status of the mute
//  4)  The minimum value for volume the control will respond to
//  5)  The maximum value for volume the control will respond to
//  6)  Whether or not the Level control will mute when it reaches minimum volume
// The volume is based on a 132-224 range, or -30dB to +16dB.  192 is used here to set
// the initial volume level to 0dB.
RPM_LEVEL       L_Lvl   = {21,192,0,132,224,0}  // Level 1 on left channel 
                                                                                                // just prior to output 1
RPM_LEVEL       R_Lvl   = {26,192,0,132,224,0}  // Level 6 on right channel 
                                                                                                // just prior to output 2  

// These 1 of N Select Types take three elements. The first is the PID which is assigned
// by the RPM unit, second comes the Number of Channels that can be selected, and third 
// is the currently selected channel.
RPM_SELECT      L_SEL   = {31,3,1}      // 3 channel select for output 1 
RPM_SELECT      R_SEL   = {33,4,1}      // 4 channel select for output 2
RPM_SELECT      CH3_SEL = {36,4,1}      // 4 channel select for output 2
RPM_SELECT      CH4_SEL = {37,4,1}      // 4 channel select for output 2


integer currPset, zone1Source, zone2Source, zone3Source

//  ***********  LATCHING DEFINITIONS GO BELOW  **********
DEFINE_LATCHING



// *******  MUTUALLY EXCLUSIVE DEFINITIONS GO BELOW ******
DEFINE_MUTUALLY_EXCLUSIVE


// ******  SUBROUTINE/FUNCTION DEFINITIONS GO BELOW  *****

// This Call gets the status of each 'Select' and 'Level'.  See the explanation
// in the Include file for each function call.
Define_Call 'UpdateStatus'
{
    // Use the GetRPMSelect() function to get the currently selected input 
    // of a 1-of-N Select.  Pass the appropriate Select, (defined in the Variable section), 
    // as an argument to the function.
    if(GetRPMSelect(L_SEL))
        zone1Source = L_SEL.currChan + 6    // +6 compensates for button # of touch panel vs. 
                                                                            // actual input #
    if(GetRPMSelect(CH3_SEL))
        zone2Source = CH3_SEL.currChan + 9
    if(GetRPMSelect(CH4_SEL))
        zone3Source = CH4_SEL.currChan + 12

    // Use the GetRPMLevel() function to get the current level value of a Level object
    // Pass the appropriate Level, (defined in the Variable section), and a function 
    // constant as arguments to the function.
    // See the explanation in the 'raneRPM.axi' file
    GetRPMLevel(L_LVL, RPM_MUTE)
    GetRPMLevel(R_LVL, RPM_MUTE)
    if(GetRPMLevel(L_LVL, RPM_INCREMENT))
    Send_Level dvPNL,L_LEVELBAR,L_LVL.vol    
    if(GetRPMLevel(R_LVL, RPM_INCREMENT))
    Send_Level dvPNL,R_LEVELBAR,R_LVL.vol    
    if(L_LVL.vol <> R_LVL.vol) {
        off[chanLink]
        Send_Command dvPNL, "'TEXT1-Link'"
    }
}


// *************  STARTUP CODE GOES BELOW  ***************

DEFINE_START

Call 'UpdateStatus'

//  *************  THE EVENTS GOES BELOW  ****************

DEFINE_EVENT
//  ****************  Data Events  **************************


//  ****************  Button Events  *************************

Button_Event[dvPNL, 16]    // recall preset 1
Button_Event[dvPNL, 17]    // recall preset 2
Button_Event[dvPNL, 18]    // recall preset 3
Button_Event[dvPNL, 19]    // recall preset 4
{
  Push: {
    Local_Var chan
    
    chan = button.input.channel
    // See 'raneRPM.axi' for function description of RecallRPMPreset()
    if(RecallRPMPreset(button.input.channel - 15))
      currPset = chan
      
    wait 5 Call 'UpdateStatus'
  }
}


Button_Event[dvPNL, 1]    // mute left output level
{
  Push: {
        currPset = 0                // turn off any presets on display - this affects only the display
                                                // no command is sent to the RPM unit
    if(!L_LVL.mute) {   // not muted - so mute

            // Use the SetRPMLevel() function to set the current level value of a Level object
            // Pass the appropriate Level,(defined in the Variable section), and a function constant
            // as arguments to the function.
            // See the explanation in the 'raneRPM.axi' file
      if(SetRPMLevel(L_LVL, RPM_MUTE)) {
        on[dvPNL,bLMUTE]
      }
      if(chanLink) {
          if(SetRPMLevel(R_LVL, RPM_MUTE)) {
          on[dvPNL,bRMUTE]
        }
      }
    }
    else {    // unmute
      if(SetRPMLevel(L_LVL, RPM_UNMUTE)) {
        off[dvPNL,bLMUTE]
      }
      if(chanLink) {
          if(SetRPMLevel(R_LVL, RPM_UNMUTE)) {
          off[dvPNL,bRMUTE]
        }
      }
    }
  }
}

Button_Event[dvPNL, 2] // mute chan 2 level
{
  Push: {
        currPset = 0
    if(!R_LVL.mute) {   // not muted - so mute
      if(SetRPMLevel(R_LVL, RPM_MUTE)) {
        on[dvPNL,bRMUTE]
      }
      if(chanLink) {
          if(SetRPMLevel(L_LVL, RPM_MUTE)) {
          on[dvPNL,bLMUTE]
        }
      }
    }
    else {    // unmute
      if(SetRPMLevel(R_LVL, RPM_UNMUTE)) {
        off[dvPNL,bRMUTE]
      }
      if(chanLink) {
          if(SetRPMLevel(L_LVL, RPM_UNMUTE)) {
          off[dvPNL,bLMUTE]
        }
      }
    }
  }
}

Button_Event[dvPNL, 3] // increment chan 1 level
{
  Push: {
        currPset = 0
    if(SetRPMLevel(L_LVL, RPM_INCREMENT))
      Send_Level dvPNL,L_LEVELBAR,L_LVL.vol    
    if(chanLink) {
        R_LVL.vol = L_LVL.vol
        if(SetRPMLevel(R_LVL, RPM_ABSOLUTE))
          Send_Level dvPNL,R_LEVELBAR,R_LVL.vol
    }
  }
  Hold[1,Repeat]: {
    if(SetRPMLevel(L_LVL, RPM_INCREMENT))
      Send_Level dvPNL,L_LEVELBAR,L_LVL.vol    
    if(chanLink) {
        R_LVL.vol = L_LVL.vol
        if(SetRPMLevel(R_LVL, RPM_ABSOLUTE))
          Send_Level dvPNL,R_LEVELBAR,R_LVL.vol
    }
  }
}

Button_Event[dvPNL, 5] // decrement chan 1 level
{
  Push: {
        currPset = 0
    if(SetRPMLevel(L_LVL, RPM_DECREMENT))
      Send_Level dvPNL,L_LEVELBAR,L_LVL.vol
    if(chanLink) {
        R_LVL.vol = L_LVL.vol
        if(SetRPMLevel(R_LVL, RPM_ABSOLUTE))
          Send_Level dvPNL,R_LEVELBAR,R_LVL.vol
    }
  }
  Hold[1,Repeat]: {
    if(SetRPMLevel(L_LVL, RPM_DECREMENT))
      Send_Level dvPNL,L_LEVELBAR,L_LVL.vol
    if(chanLink) {
        R_LVL.vol = L_LVL.vol
        if(SetRPMLevel(R_LVL, RPM_ABSOLUTE))
          Send_Level dvPNL,R_LEVELBAR,R_LVL.vol
    }
  }
}

Button_Event[dvPNL, 4] // increment chan 2 level
{
  Push: {
        currPset = 0
    if(SetRPMLevel(R_LVL, RPM_INCREMENT))
      Send_Level dvPNL,R_LEVELBAR,R_LVL.vol    
    if(chanLink) {
        L_LVL.vol = R_LVL.vol
        if(SetRPMLevel(L_LVL, RPM_ABSOLUTE))
          Send_Level dvPNL,L_LEVELBAR,L_LVL.vol
    }
  }
  Hold[1,Repeat]: {
    if(SetRPMLevel(R_LVL, RPM_INCREMENT))
      Send_Level dvPNL,R_LEVELBAR,R_LVL.vol    
    if(chanLink) {
        L_LVL.vol = R_LVL.vol
        if(SetRPMLevel(L_LVL, RPM_ABSOLUTE))
          Send_Level dvPNL,L_LEVELBAR,L_LVL.vol
    }
  }
}

Button_Event[dvPNL, 6] // decrement chan 2 level
{
  Push: {
        currPset = 0
    if(SetRPMLevel(R_LVL, RPM_DECREMENT))
      Send_Level dvPNL,R_LEVELBAR,R_LVL.vol
    if(chanLink) {
        L_LVL.vol = R_LVL.vol
        if(SetRPMLevel(L_LVL, RPM_ABSOLUTE))
          Send_Level dvPNL,L_LEVELBAR,L_LVL.vol
    }
  }
  Hold[1,Repeat]: {
    if(SetRPMLevel(R_LVL, RPM_DECREMENT))
      Send_Level dvPNL,R_LEVELBAR,R_LVL.vol
    if(chanLink) {
        L_LVL.vol = R_LVL.vol
        if(SetRPMLevel(L_LVL, RPM_ABSOLUTE))
          Send_Level dvPNL,L_LEVELBAR,L_LVL.vol
    }
  }
}

Button_Event[dvPNL, 7] // select CD on ch. 1
Button_Event[dvPNL, 8] // select Cass on ch. 1
Button_Event[dvPNL, 9] // select Tone on ch. 1
{
  Push: {
    Local_Var chan

    // To use the 1-of-N Select block, set the .currChan field of a Select block to the input 
    // channel you want selected, then call the SetRPMSelect() function.  Pass a previously 
    // defined 1-of-N Select variable as the argument to the function.

        currPset = 0
    chan = button.input.channel
    L_SEL.currChan = chan - 6
    R_SEL.currChan = chan - 6
    
    if(SetRPMSelect(L_SEL) && SetRPMSelect(R_SEL))
      zone1Source = chan  
  }
}

Button_Event[dvPNL, 10] // select CD on ch. 3
Button_Event[dvPNL, 11] // select Cass on ch. 3
Button_Event[dvPNL, 12] // select Tone on ch. 3
{
  Push: {
    Local_Var chan

        currPset = 0
    chan = button.input.channel
    CH3_SEL.currChan = chan - 9
    if(SetRPMSelect(CH3_SEL))
      zone2Source = chan  
  }
}

Button_Event[dvPNL, 13] // select CD on ch. 4
Button_Event[dvPNL, 14] // select Cass on ch. 4
Button_Event[dvPNL, 15] // select Tone on ch. 4
{
  Push: {
    Local_Var chan

        currPset = 0
    chan = button.input.channel
    CH4_SEL.currChan = chan - 12
    if(SetRPMSelect(CH4_SEL))
      zone3Source = chan  
  }
}

Button_Event[dvPNL, 20] // link channels 1 & 2
{
  Push: {
        chanLink = !chanLink
        if(chanLink) {
            // When linking channels, we want to set the mute and/or volume levels
            // to be the same on both channels.  Here we use the left channel as
            // the master reference.
            Send_Command dvPNL, "'TEXT1-UnLink'"
            if(R_LVL.mute <> L_LVL.mute) {
                if(L_LVL.mute)
                    SetRPMLevel(R_LVL,RPM_MUTE)
                else
                    SetRPMLevel(R_LVL,RPM_UNMUTE)
            }
            if(R_LVL.vol <> L_LVL.vol) {
                R_LVL.vol = L_LVL.vol
                SetRPMLevel(R_LVL,RPM_ABSOLUTE)
            }
            Call 'UpdateStatus'
        }
        else
            Send_Command dvPNL, "'TEXT1-Link'"
    }
}

//  ***********  THE ACTUAL PROGRAM GOES BELOW  **********

DEFINE_PROGRAM

// This section is to set buton status on the touch panel
// source for zone 1
[dvPNL,7] = (zone1Source == 7)
[dvPNL,8] = (zone1Source == 8)
[dvPNL,9] = (zone1Source == 9)

// mute status for channels 1 & 2
[dvPNL,1] = (L_LVL.mute)
[dvPNL,2] = (R_LVL.mute)

// source for zone 2
[dvPNL,10] = (zone2Source == 10)
[dvPNL,11] = (zone2Source == 11)
[dvPNL,12] = (zone2Source == 12)

// source for zone 3
[dvPNL,13] = (zone3Source == 13)
[dvPNL,14] = (zone3Source == 14)
[dvPNL,15] = (zone3Source == 15)

// preset selection
[dvPNL,16] = (currPset == 16)
[dvPNL,17] = (currPset == 17)
[dvPNL,18] = (currPset == 18)
[dvPNL,19] = (currPset == 19)

// channels linked or not
[dvPNL,20] = (chanLink)

//  ****************   END OF PROGRAM  *******************

